"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const child_process_1 = require("child_process");
const Table = require("cli-table2");
const path = require("path");
const readline = require("readline");
const run_1 = require("../base/run");
const cache_1 = require("../services/cache");
const conf_1 = require("../services/conf");
const docker_1 = require("../services/docker");
const install_1 = require("../services/install");
const logger_1 = require("../services/logger");
const validate_1 = require("../services/validate");
const { resolve, join } = path;
const { RdTypes, cwd, rdcConfName } = conf_1.default;
class Run extends run_1.default {
    constructor() {
        super(...arguments);
        this.rebuild = false;
        this.install = false;
    }
    get from() {
        if (conf_1.default.isApp) {
            const { container } = conf_1.default.getAppConf();
            return container.name;
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConfPath = resolve(cwd, rdcConfName);
            const rdcConf = require(rdcConfPath);
            return rdcConf.nodeVersion || 'node:latest';
        }
    }
    get tag() {
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConf = require(join(conf_1.default.cwd, rdcConfName));
            return rdcConf.docker.tag.split(':')[0];
        }
        if (conf_1.default.isApp) {
            // name app image with project dir name
            return path.basename(conf_1.default.cwd);
        }
    }
    get workDir() {
        const { dockerWorkDirRoot } = conf_1.default;
        let name;
        if (conf_1.default.isApp) {
            const { container } = conf_1.default.getAppConf();
            name = container.name.split(':')[0];
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConfPath = resolve(cwd, rdcConfName);
            const rdcConf = require(rdcConfPath);
            const { tag } = rdcConf.docker;
            name = tag.split(':')[0];
        }
        return `${dockerWorkDirRoot}/${name}`;
    }
    get ports() {
        if (conf_1.default.isApp) {
            const { docker } = conf_1.default.getAppConf();
            return docker.ports || [];
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConf = require(join(conf_1.default.cwd, conf_1.default.rdcConfName));
            return rdcConf.docker.ports || [];
        }
    }
    async preInit() {
        await super.preInit();
        await this.config.runHook('checkUpdate', {});
        const { flags } = this.parse(Run);
        this.rebuild = flags.rebuild;
        this.install = flags.install;
        if (conf_1.default.rdType === RdTypes.Container) {
            await validate_1.validateRdc();
        }
        if (conf_1.default.isApp) {
            await validate_1.validateRda();
        }
    }
    async preRun() {
        await docker_1.default.genDockerFile(this.workDir, this.from, conf_1.default.localCacheDir, conf_1.default.isApp);
        await docker_1.default.genDockerCompose(this.workDir, this.cmd, this.ports, this.watch, `dev-${this.tag}`, conf_1.default.localCacheDir, conf_1.default.isApp, this.cmd === 'build');
        if (conf_1.default.isApp) {
            const cacheContainer = cache_1.default.get('container');
            const { container } = conf_1.default.getAppConf();
            if (cacheContainer && cacheContainer !== container.name) {
                const table = new Table({
                    style: {
                        'padding-left': 0,
                        'padding-right': 0,
                        border: ['yellow']
                    },
                    colWidths: [50],
                    rowHeights: [2],
                });
                table.push([{
                        hAlign: 'center',
                        content: `Container updated: ${container.name}\nPlease run: $rde install`
                    }]);
                // tslint:disable:no-console
                console.log(table.toString());
            }
            cache_1.default.set('container', container.name);
            await install_1.default.app({
                skipInstall: true,
            });
        }
    }
    async run() {
        // not using docker-compose cuz .dockerignore in sub dir is not working,
        // build with docker-compose is slow if node_modules exists
        await docker_1.default.build(`dev-${this.tag}`, cwd, this.rebuild, `${conf_1.default.localCacheDir}/Dockerfile`);
        let args = ['run', '--rm', '--service-ports', 'rde', 'rde', 'docker:run', this.cmd];
        logger_1.debug(`docker build dev-${this.tag}`);
        if (this.watch) {
            args.push('--watch');
        }
        if (this.verbose) {
            args.push('-v');
        }
        if (this.extras) {
            args = args.concat(['-e', this.extras]);
        }
        let child = null;
        process.on('SIGINT', () => child.kill());
        if (this.watch) {
            const rl = readline.createInterface({
                input: process.stdin,
                output: process.stdout
            });
            rl.on('SIGINT', () => {
                child.kill();
                process.exit(0);
            });
        }
        logger_1.debug(`docker-compose ${args.join(' ')}`);
        child = child_process_1.spawn('docker-compose', args, {
            cwd: conf_1.default.localCacheDir,
            stdio: 'inherit',
        });
        child.on('close', code => {
            if (code !== 0) {
                process.exit(code);
            }
        });
    }
}
Run.description = 'run scripts provided by container';
Run.examples = [
    '$ rde run <cmd>',
];
Run.flags = Object.assign({}, run_1.default.flags, { rebuild: command_1.flags.boolean({
        char: 'r',
        description: 'rebuild image before run',
    }), install: command_1.flags.boolean({
        char: 'i',
        description: 'generate local node_modules',
    }) });
Run.args = [{
        name: 'cmd',
        required: false,
        description: 'scripts provided by container',
    }];
exports.default = Run;
